@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | uopz                                                                 |
   +----------------------------------------------------------------------+
-  | Copyright (c) Joe Watkins 2016-2020                                  |
+  | Copyright (c) Joe Watkins 2016-2021                                  |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -33,10 +33,10 @@ zend_bool uopz_set_return(zend_class_entry *clazz, zend_string *name, zval *valu
 	HashTable *returns;
 	uopz_return_t ret;
 	zend_string *key = zend_string_tolower(name);
-	zend_function *function;
 
 	if (clazz) {
-		if (uopz_find_method(clazz, key, &function) != SUCCESS) {
+		zend_function *function = uopz_find_method(clazz, key);
+		if (!function) {
 			uopz_exception(
 				"failed to set return for %s::%s, the method does not exist",
 				ZSTR_VAL(clazz->name),
@@ -126,11 +126,13 @@ uopz_return_t* uopz_find_return(zend_function *function) { /* {{{ */
 	uopz_return_t *ureturn;
 	HashTable *returns;
 
-	if (function->common.fn_flags & ZEND_ACC_CLOSURE) {
+	if ((function == NULL) ||
+		(function->common.function_name == NULL) ||
+		(function->common.fn_flags & ZEND_ACC_CLOSURE)) {
 		return NULL;
 	}
 
-	if (!function->common.function_name) {
+	if (EG(flags) & EG_FLAGS_IN_SHUTDOWN) {
 		return NULL;
 	}
 
@@ -142,8 +144,8 @@ uopz_return_t* uopz_find_return(zend_function *function) { /* {{{ */
 
 	if (!returns) {
 		if (function->common.prototype && 
-		    function->common.prototype->common.scope &&
-		    function->common.prototype->common.scope->ce_flags & ZEND_ACC_INTERFACE) {
+			function->common.prototype->common.scope &&
+			function->common.prototype->common.scope->ce_flags & ZEND_ACC_INTERFACE) {
 			return uopz_find_return(
 				function->common.prototype);
 		}
@@ -158,8 +160,6 @@ uopz_return_t* uopz_find_return(zend_function *function) { /* {{{ */
 	return ureturn;
 } /* }}} */
 
-extern PHP_FUNCTION(php_call_user_func);
-
 void uopz_execute_return(uopz_return_t *ureturn, zend_execute_data *execute_data, zval *return_value) { /* {{{ */
 	zend_fcall_info fci = empty_fcall_info;
 	zend_fcall_info_cache fcc = empty_fcall_info_cache;
@@ -172,7 +172,7 @@ void uopz_execute_return(uopz_return_t *ureturn, zend_execute_data *execute_data
 
 	ureturn->flags ^= UOPZ_RETURN_BUSY;
 
-	zend_create_closure(&closure, (zend_function*) zend_get_closure_method_def(&ureturn->value), 
+	zend_create_closure(&closure, (zend_function*) zend_get_closure_method_def(Z_OBJ(ureturn->value)), 
 		ureturn->clazz, ureturn->clazz, Z_OBJ(EX(This)) ? &EX(This) : NULL);
 
 	zend_fcall_info_init(&closure, 0, &fci, &fcc, NULL, &error);
@@ -198,6 +198,10 @@ void uopz_execute_return(uopz_return_t *ureturn, zend_execute_data *execute_data
 	}
 
 	zval_ptr_dtor(&closure);
+	
+	if (uopz_is_cufa(execute_data)) {
+		zend_fcall_info_args_clear(&fci, 1);
+	}
 
 	ureturn->flags ^= UOPZ_RETURN_BUSY;
 } /* }}} */
