@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | uopz                                                                 |
   +----------------------------------------------------------------------+
-  | Copyright (c) Joe Watkins 2016-2020                                  |
+  | Copyright (c) Joe Watkins 2016-2021                                  |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -50,7 +50,7 @@ ZEND_DECLARE_MODULE_GLOBALS(uopz)
 
 PHP_INI_BEGIN()
 	STD_PHP_INI_ENTRY("uopz.disable", "0", PHP_INI_SYSTEM, OnUpdateBool, disable, zend_uopz_globals, uopz_globals)
-	STD_PHP_INI_ENTRY("uopz.exit",    "0", PHP_INI_SYSTEM, OnUpdateBool, exit,    zend_uopz_globals, uopz_globals)
+	STD_PHP_INI_ENTRY("uopz.exit",	"0", PHP_INI_SYSTEM, OnUpdateBool, exit,	zend_uopz_globals, uopz_globals)
 PHP_INI_END()
 
 /* {{{ */
@@ -128,6 +128,9 @@ static PHP_RINIT_FUNCTION(uopz)
 
 		/* disable DCE (want code after exit) */
 		level &= ~(1<<13);
+		
+		/* don't inline functions */
+		level &= ~(1<<15);
 
 		value = strpprintf(0, "0x%08X", (unsigned int) level);
 
@@ -256,18 +259,16 @@ static PHP_FUNCTION(uopz_get_return)
 /* {{{ proto void uopz_set_mock(string class, mixed mock) */
 static PHP_FUNCTION(uopz_set_mock) 
 {
-	zend_string *clazz = NULL;
-	zval *mock = NULL;
+	zend_string *clazz;
+	zval *mock;
 
 	uopz_disabled_guard();
 
-	if (uopz_parse_parameters("Sz", &clazz, &mock) != SUCCESS) {
-		uopz_refuse_parameters(
-			"unexpected parameter combination, expected (class, mock), classes not found ?");
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Sz", &clazz, &mock) != SUCCESS) {
+		RETURN_THROWS();
 	}
 
-	if (!mock || (Z_TYPE_P(mock) != IS_STRING && Z_TYPE_P(mock) != IS_OBJECT)) {
+	if (Z_TYPE_P(mock) != IS_STRING && Z_TYPE_P(mock) != IS_OBJECT) {
 		uopz_refuse_parameters(
 			"unexpected parameter combination, mock is expected to be a string, or an object");
 		return;
@@ -279,14 +280,12 @@ static PHP_FUNCTION(uopz_set_mock)
 /* {{{ proto void uopz_unset_mock(string mock) */
 static PHP_FUNCTION(uopz_unset_mock) 
 {
-	zend_string *clazz = NULL;
+	zend_string *clazz;
 
 	uopz_disabled_guard();
 
-	if (uopz_parse_parameters("S", &clazz) != SUCCESS) {
-		uopz_refuse_parameters(
-			"unexpected parameter combination, expected (clazz), class not found ?");
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "S", &clazz) != SUCCESS) {
+		RETURN_THROWS();
 	}
 
 	uopz_unset_mock(clazz);
@@ -295,14 +294,12 @@ static PHP_FUNCTION(uopz_unset_mock)
 /* {{{ proto void uopz_get_mock(string mock) */
 static PHP_FUNCTION(uopz_get_mock) 
 {
-	zend_string *clazz = NULL;
+	zend_string *clazz;
 
 	uopz_disabled_guard();
 
-	if (uopz_parse_parameters("S", &clazz) != SUCCESS) {
-		uopz_refuse_parameters(
-			"unexpected parameter combination, expected (clazz), class not found ?");
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "S", &clazz) != SUCCESS) {
+		RETURN_THROWS();
 	}
 
 	uopz_get_mock(clazz, return_value);
@@ -505,42 +502,8 @@ static PHP_FUNCTION(uopz_undefine)
 	}
 } /* }}} */
 
-/* {{{ proto bool uopz_implement(string class, string interface) */
-static PHP_FUNCTION(uopz_implement)
-{
-	zend_class_entry *clazz = NULL;
-	zend_class_entry *interface = NULL;
-
-	uopz_disabled_guard();
-
-	if (uopz_parse_parameters("CC", &clazz, &interface) != SUCCESS) {
-		uopz_refuse_parameters(
-			"unexpected parameter combination, expected (class, interface)");
-		return;
-	}
-
-	RETURN_BOOL(uopz_implement(clazz, interface));
-} /* }}} */
-
-/* {{{ proto bool uopz_extend(string class, string parent) */
-static PHP_FUNCTION(uopz_extend)
-{
-	zend_class_entry *clazz = NULL;
-	zend_class_entry *parent = NULL;
-
-	uopz_disabled_guard();
-
-	if (uopz_parse_parameters("CC", &clazz, &parent) != SUCCESS) {
-		uopz_refuse_parameters(
-			"unexpected parameter combination, expected (class, parent)");
-		return;
-	}
-
-	RETURN_BOOL(uopz_extend(clazz, parent));
-} /* }}} */
-
 /* {{{ proto int uopz_flags(string function [, int flags])
-       proto int uopz_flags(string class, string function [, int flags]) */
+	   proto int uopz_flags(string class, string function [, int flags]) */
 static PHP_FUNCTION(uopz_flags)
 {
 	zend_string *name = NULL;
@@ -564,16 +527,17 @@ static PHP_FUNCTION(uopz_flags)
 			 void uopz_set_property(string class, string property, mixed value) */
 static PHP_FUNCTION(uopz_set_property) 
 {
-	zval *scope = NULL;
-	zval *prop  = NULL;
-	zval *value = NULL;
+	zval *scope;
+	zend_string *prop;
+	zval *value;
 
 	uopz_disabled_guard();
 
-	if (uopz_parse_parameters("zzz", &scope, &prop, &value) != SUCCESS ||
-		!scope || !prop || !value ||
-		(Z_TYPE_P(scope) != IS_OBJECT && Z_TYPE_P(scope) != IS_STRING) ||
-		Z_TYPE_P(prop) != IS_STRING) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "zSz", &scope, &prop, &value) != SUCCESS) {
+		RETURN_THROWS();
+	}
+
+	if (Z_TYPE_P(scope) != IS_OBJECT && Z_TYPE_P(scope) != IS_STRING) {
 		uopz_refuse_parameters(
 			"unexpected paramter combination, expected "
 			"(class, property, value) or (object, property, value)");
@@ -589,22 +553,23 @@ static PHP_FUNCTION(uopz_set_property)
 			return;
 		}
 
-		uopz_set_static_property(ce, Z_STR_P(prop), value);
+		uopz_set_static_property(ce, prop, value);
 	}
 } /* }}} */
 
 /* {{{ proto mixed uopz_get_property(object instance, string property) 
 	   proto mixed uopz_get_property(string class, string property) */
 static PHP_FUNCTION(uopz_get_property) {
-	zval *scope = NULL;
-	zval *prop  = NULL;
+	zval *scope;
+	zend_string *prop;
 
 	uopz_disabled_guard();
 
-	if (uopz_parse_parameters("zz", &scope, &prop) != SUCCESS ||
-		!scope || !prop ||
-		(Z_TYPE_P(scope) != IS_OBJECT && Z_TYPE_P(scope) != IS_STRING) ||
-		Z_TYPE_P(prop) != IS_STRING) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "zS", &scope, &prop) != SUCCESS) {
+		RETURN_THROWS();
+	}
+
+	if (Z_TYPE_P(scope) != IS_OBJECT && Z_TYPE_P(scope) != IS_STRING) {
 		uopz_refuse_parameters(
 			"unexpected paramter combination, expected "
 			"(class, property) or (object, property)");
@@ -620,7 +585,7 @@ static PHP_FUNCTION(uopz_get_property) {
 			return;
 		}
 
-		uopz_get_static_property(ce, Z_STR_P(prop), return_value);
+		uopz_get_static_property(ce, prop, return_value);
 	}
 } /* }}} */
 
@@ -629,6 +594,10 @@ static PHP_FUNCTION(uopz_get_exit_status) {
 
 	uopz_disabled_guard();
 
+	if (zend_parse_parameters_none() != SUCCESS) {
+		RETURN_THROWS();
+	}
+
 	if (Z_TYPE(UOPZ(estatus)) != IS_UNDEF) {
 		ZVAL_COPY(return_value, &UOPZ(estatus));
 	}
@@ -636,14 +605,12 @@ static PHP_FUNCTION(uopz_get_exit_status) {
 
 /* {{{ proto mixed uopz_allow_exit(bool allow) */
 static PHP_FUNCTION(uopz_allow_exit) {
-	zend_bool allow = 0;
+	zend_bool allow;
 
 	uopz_disabled_guard();
 	
-	if (uopz_parse_parameters("b", &allow) != SUCCESS) {
-		uopz_refuse_parameters(
-			"unexpected parameter combination, expected (allow)");
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "b", &allow) != SUCCESS) {
+		RETURN_THROWS();
 	}
 
 	UOPZ(exit) = allow;
@@ -651,7 +618,15 @@ static PHP_FUNCTION(uopz_allow_exit) {
 
 /* {{{ uopz_functions[]
  */
-#define UOPZ_FE(f) PHP_FE(f, NULL)
+ZEND_BEGIN_ARG_INFO(uopz_ignore_arginfo, 1)
+	ZEND_ARG_VARIADIC_INFO(0, arguments)
+ZEND_END_ARG_INFO()
+# define UOPZ_FE(f) PHP_FE(f, uopz_ignore_arginfo)
+
+ZEND_BEGIN_ARG_INFO(uopz_no_args_arginfo, 0)
+ZEND_END_ARG_INFO()
+# define UOPZ_FE_NOARGS(f) PHP_FE(f, uopz_no_args_arginfo)
+
 static const zend_function_entry uopz_functions[] = {
 	UOPZ_FE(uopz_set_return)
 	UOPZ_FE(uopz_get_return)
@@ -666,14 +641,12 @@ static const zend_function_entry uopz_functions[] = {
 	UOPZ_FE(uopz_unset_hook)
 	UOPZ_FE(uopz_add_function)
 	UOPZ_FE(uopz_del_function)
-	UOPZ_FE(uopz_extend)
-	UOPZ_FE(uopz_implement)
 	UOPZ_FE(uopz_flags)
 	UOPZ_FE(uopz_redefine)
 	UOPZ_FE(uopz_undefine)
 	UOPZ_FE(uopz_set_property)
 	UOPZ_FE(uopz_get_property)
-	UOPZ_FE(uopz_get_exit_status)
+	UOPZ_FE_NOARGS(uopz_get_exit_status)
 	UOPZ_FE(uopz_allow_exit)
 
 	UOPZ_FE(uopz_call_user_func)
